texrender
===========

A small AO renderer that showcases our texture space caching and reconstruction system.   
*Tested on Visual Studio 2015, x64 only*. 

The code is largely based on [Embree's path tracing tutorial](https://github.com/embree/embree)
where we have added a texture space cache and simple reconstruction filter for an AO example.

# Installation
* Download and install the [Embree](https://embree.github.io/) ray tracing kernels.   Tested with Embree version 2.11, Win x64 binaries.
* Add an environment variable `EMBREE_INSTALL_DIR`, to the embree installation location, typically `C:\Program Files\Intel\Embree v2.11.0 x64`
* The Visual Studio solution file is located in `texrender.sln`.
* Verify that the embree kernels are set in the VC++ Directories 
`Properties->Configuration Properties->VC++ Directories`
	* **Include Directories** : `$(EMBREE_INSTALL_DIR)\include;` 
	* **Library Directories**: `$(EMBREE_INSTALL_DIR)\lib;`
* Build the x64 Release target and run. The filter size and mip bias can be adjusted in the simple GUI. The filter is set to 4.0 by default.

# External dependencies
* [Embree](https://embree.github.io/) ray tracing kernels. *Note that the Embree Win x64 binaries are not included in the texrender source, but need to be downloaded and installed separately.*
Embree is released as open source under the
[Apache 2.0 license](http://www.apache.org/licenses/LICENSE-2.0).
* [LodePNG](http://lodev.org/lodepng/) included. Free license. 
* [ImGui](https://github.com/ocornut/imgui) included. Bloat-free Immediate Mode Graphical User interface for C++ with minimal dependencies. MIT licence.
* [UVChecker-map](https://github.com/Arahnoid/UVChecker-map) One png texture is included. Public domain.
* [pbrt-v2](https://github.com/mmp/pbrt-v2) . The code for ray differential computation is taken from the pbrt source code. BSD license.

# Example render 
![preview](screenshot.jpg)

# Code organization

The per-pixel render function is located in `renderer/renderer.cpp` in the function `RenderPath`. 

The AO term is either computed directly, or if the cache is enabled, the AO term is lazily evaluated and filtered through the texture space cache, which is implemented in `renderer/patchtexture.[h,cpp]`.

The texture space cache triggers a `texelCallback`, which evaluates the AO term
at a quantized position in texture space. We use the embree `rtcInterpolate` call
to retrieve the surface properties (position, gradients) at the quantized uv.

To keep this code example easy to follow, we only include an **unlimited cache**, in `patchtexture.[h,cpp]`. 
For each face of a subdivision mesh, a mip map hierarchy for samples and texels are currently preallocated. In a production setting, this should be replaced by a limited-sized cache.



