// ======================================================================== //
// Copyright 2009-2016 Intel Corporation                                    //
//                                                                          //
// Licensed under the Apache License, Version 2.0 (the "License");          //
// you may not use this file except in compliance with the License.         //
// You may obtain a copy of the License at                                  //
//                                                                          //
//     http://www.apache.org/licenses/LICENSE-2.0                           //
//                                                                          //
// Unless required by applicable law or agreed to in writing, software      //
// distributed under the License is distributed on an "AS IS" BASIS,        //
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. //
// See the License for the specific language governing permissions and      //
// limitations under the License.                                           //
//																			//
// Modified by Jacob Munkberg												//
// ======================================================================== //

#include "image.h"

#include <iostream>
#include <cstring>
#include <cstdio>
#include "lodepng\lodepng.h"

namespace embree
{
	void storePNG(const Ref<Image>& img, const FileName& fileName, bool sRGB)
	{
		uint8_t* image = new uint8_t[img->height*img->width * 4];
		if (sRGB)
			img->convertToRGBA8_sRGBA(image);
		else
			img->convertToRGBA8(image);
		lodepng::encode(fileName, image, img->width, img->height);
		delete[] image;
	}

	void storePNG_buffer(uint8_t* image, int width, int height, const FileName& fileName)
	{
		lodepng::encode(fileName, image, width, height);
	}

	/*! read PNG file from disk */
	Ref<Image> loadPNG(const FileName& fileName)
	{
		std::vector<uint8_t> image; //the raw pixels
		unsigned int width, height;

		//load and decode
		unsigned error = lodepng::decode(image, width, height, fileName);

		//if there's an error, display it
		if (error) std::cout << "decoder error " << error << ": " << lodepng_error_text(error) << std::endl;

		int channels = 4;

		Ref<Image> img = new Image4uc(width, height, fileName);

		for (size_t y = 0; y<height; y++)
		{
			for (size_t x = 0; x<width; x++)
			{
				float col[4];
				for (int c = 0; c < channels; c++) {
					uint8_t val = image[channels*(x + y*width) + c];
					col[c] = val / 255.0f;
				}

				Color4 c(col[0], col[1], col[2], 1.0f);
				img.ptr->set(x, y, c);
			}
		}

		return img;
	}
}


