// ======================================================================== //
// Copyright 2009-2016 Intel Corporation                                    //
//                                                                          //
// Licensed under the Apache License, Version 2.0 (the "License");          //
// you may not use this file except in compliance with the License.         //
// You may obtain a copy of the License at                                  //
//                                                                          //
//     http://www.apache.org/licenses/LICENSE-2.0                           //
//                                                                          //
// Unless required by applicable law or agreed to in writing, software      //
// distributed under the License is distributed on an "AS IS" BASIS,        //
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. //
// See the License for the specific language governing permissions and      //
// limitations under the License.                                           //
//																			//
// Modified by Jacob Munkberg												//
// ======================================================================== //

#pragma once

#include "differentialgeometry.h"
#include "sample.h"
#include "shapesampler.h"
#include "optics.h"
#include "../../common/math/vec2.h"

namespace embree {

	struct BRDF {
		virtual Vec3fa eval(const Vec3fa &wo, const DifferentialGeometry &dg, const Vec3fa &wi) = 0;
		virtual Vec3fa sample(const Vec3fa &wo, const DifferentialGeometry &dg, Sample3f &wi, const Vec2f &s) = 0;
	};

	////////////////////////////////////////////////////////////////////////////////
	//                                Lambertian BRDF                             //
	////////////////////////////////////////////////////////////////////////////////
	struct Lambertian : public BRDF
	{
		Vec3fa R;

		Lambertian(const Vec3fa& _r) : R(_r) {}

		inline Vec3fa eval(const Vec3fa &wo, const DifferentialGeometry &dg, const Vec3fa &wi) override
		{
			return R * (1.0f / (float)(float(pi))) * clamp(dot(wi, dg.Ns));
		}

		inline Vec3fa sample(const Vec3fa &wo,
			const DifferentialGeometry &dg,
			Sample3f &wi,
			const Vec2f &s) override
		{
			wi = cosineSampleHemisphere(s.x, s.y, dg.Ns);
			return eval(wo, dg, wi.v);
		}
	};
}