// ======================================================================== //
// Copyright 2009-2016 Intel Corporation                                    //
//                                                                          //
// Licensed under the Apache License, Version 2.0 (the "License");          //
// you may not use this file except in compliance with the License.         //
// You may obtain a copy of the License at                                  //
//                                                                          //
//     http://www.apache.org/licenses/LICENSE-2.0                           //
//                                                                          //
// Unless required by applicable law or agreed to in writing, software      //
// distributed under the License is distributed on an "AS IS" BASIS,        //
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. //
// See the License for the specific language governing permissions and      //
// limitations under the License.                                           //
//																			//
// Modified by Jacob Munkberg												//
// ======================================================================== //

#pragma once

#include "../../common/math/vec3.h"
#include "shapesampler.h"
#include "ray.h"

namespace embree
{
	inline Vec3fa face_forward(const Vec3fa& dir, const Vec3fa& _Ng) {
		const Vec3fa Ng = _Ng;
		return dot(dir, Ng) < 0.0f ? Ng : neg(Ng);
	}

	struct DifferentialGeometry
	{
		int geomID;
		int primID;
		float u, v;
		Vec3fa P;
		Vec3fa Ng;
		Vec3fa Ns;
		Vec3fa Tx; //direction along hair
		Vec3fa Ty;
		float tnear_eps;

		DifferentialGeometry() {}

		DifferentialGeometry(const RTCRay& ray, const Vec3fa& N)
		{
			geomID = ray.geomID;
			primID = ray.primID;
			u = ray.u;
			v = ray.v;
			P = ray.org + ray.tfar*ray.dir;
			Ng = face_forward(ray.dir, normalize(ray.Ng));
			Ns = face_forward(ray.dir, normalize(N));
		}
	};
}
