// ======================================================================== //
// Copyright 2009-2016 Intel Corporation                                    //
//                                                                          //
// Licensed under the Apache License, Version 2.0 (the "License");          //
// you may not use this file except in compliance with the License.         //
// You may obtain a copy of the License at                                  //
//                                                                          //
//     http://www.apache.org/licenses/LICENSE-2.0                           //
//                                                                          //
// Unless required by applicable law or agreed to in writing, software      //
// distributed under the License is distributed on an "AS IS" BASIS,        //
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. //
// See the License for the specific language governing permissions and      //
// limitations under the License.                                           //
//																			//
// Modified by Jacob Munkberg												//
// ======================================================================== //

#pragma once

#include "../../../common/math/vec2.h"

inline unsigned int MurmurHash3_mix(unsigned int hash, unsigned int k)
{
	const unsigned int c1 = 0xcc9e2d51;
	const unsigned int c2 = 0x1b873593;
	const unsigned int r1 = 15;
	const unsigned int r2 = 13;
	const unsigned int m = 5;
	const unsigned int n = 0xe6546b64;

	k *= c1;
	k = (k << r1) | (k >> (32 - r1));
	k *= c2;

	hash ^= k;
	hash = ((hash << r2) | (hash >> (32 - r2))) * m + n;

	return hash;
}

inline unsigned int MurmurHash3_finalize(unsigned int hash)
{
	hash ^= hash >> 16;
	hash *= 0x85ebca6b;
	hash ^= hash >> 13;
	hash *= 0xc2b2ae35;
	hash ^= hash >> 16;

	return hash;
}

inline unsigned int LCG_next(unsigned int value)
{
	const unsigned int m = 1664525;
	const unsigned int n = 1013904223;

	return value * m + n;
}

class RandomSampler
{
public:

	RandomSampler(int x, int y, int t) : s(t) {}

	void init(int pixelId, int sampleId)
	{
		unsigned int hash = s;
		hash = MurmurHash3_mix(hash, pixelId);
		hash = MurmurHash3_mix(hash, sampleId);
		hash = MurmurHash3_finalize(hash);
		s = hash;
	}

	void init(int x, int y, int sampleId)
	{
		init(x | (y << 16), sampleId);
	}

	inline float get1D()
	{
		s = LCG_next(s);
		// avoid expensive uint->float conversion
		return (float)(int)(s >> 1) * 4.656612873077392578125e-10f;
	}

	inline embree::Vec2f get2D()
	{
		float u = get1D();
		float v = get1D();
		return embree::Vec2f(u, v);
	}

protected:
	unsigned int s;
};

